// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
angular.module('taxidriver', ['ionic', 'ngCordova', 'ionic.rating'])

.run(function ($ionicPlatform, $cordovaStatusbar) {
	$ionicPlatform.ready(function () {
		// Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
		// for form inputs)
		if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
			cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
		}
		if (window.StatusBar) {
			$cordovaStatusbar.styleHex("#33CC2B");
		}
	});
})

.service('$leafletService', leafletService)
	.service('$moveJSService', moveJSService)
	.service('$apiCaller', apiCaller)
	.service('$bgGeolocService', bgGeolocationService)
	.service('$databaseService', databaseService)
	.service('$driverAppService', driverAppService)
	.service('$socketService', socketService)
	.service('$ratingService', ratingService)

.config(function ($stateProvider, $urlRouterProvider, $ionicConfigProvider, $httpProvider, $compileProvider) {

	$compileProvider.aHrefSanitizationWhitelist(/^\s*(https?|sms|tel):/);

	$httpProvider.defaults.timeout = 5000;
	$httpProvider.defaults.useXDomain = true;
	$httpProvider.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8';

	delete $httpProvider.defaults.headers.common['X-Requested-With'];
	$ionicConfigProvider.views.transition('none');

	$stateProvider

	.state('main', {
		url: "/main",
		templateUrl: "templates/mainTemplate.html",
		controller: mainController

	})
		.state('login', {
			url: "/login",
			templateUrl: "templates/loginTemplate.html",
			controller: loginController
		})
		.state('rating', {
			url: "/rating",
			templateUrl: "templates/ratingTemplate.html",
			controller: ratingController
		})

	$urlRouterProvider.otherwise('/login');
})


.filter('zpad', function () {
	return function (input, n) {
		if (input === undefined)
			input = ""
		if (input.length >= n)
			return input
		var zeros = "0".repeat(n);
		return (zeros + input).slice(-1 * n)
	};
})

.directive('onLongPress', function($timeout) {
	return {
		restrict: 'A',
		link: function($scope, $elm, $attrs) {
			$elm.bind('touchstart', function(evt) {
				// Locally scoped variable that will keep track of the long press
				$scope.longPress = true;

				// We'll set a timeout for 600 ms for a long press
				$timeout(function() {
					if ($scope.longPress) {
						// If the touchend event hasn't fired,
						// apply the function given in on the element's on-long-press attribute
						$scope.$apply(function() {
							$scope.$eval($attrs.onLongPress)
						});
					}
				}, 4600);
			});

			$elm.bind('touchend', function(evt) {
				// Prevent the onLongPress event from firing
				$scope.longPress = false;
				// If there is an on-touch-end function attached to this element, apply it
				if ($attrs.onTouchEnd) {
					$scope.$apply(function() {
						$scope.$eval($attrs.onTouchEnd)
					});
				}
			});
		}
	};
})

