/*
 * Singleton Class for requesting data to API.
 * Note that some methods are asynchronous and you should provide a callback function to
 * access the data when it is available.
 * Usage:
 *      api_caller.<method>(<arguments>)
 */
function apiCaller($http, $q) {


    /**
     * Get/Set API url.
     */
    this.domain = 'https://mgetaxi.com';
    //this.domain = 'http://192.168.254.101';l

    this.backgroundGeolocationUrl = 'http://services.mgetaxi.com/locationfeed.php';

    this.api_url = this.domain + '/api/v1/';

    this.register_url = function (device_id) {
        r_arguments = {
            'device_id': device_id
        };
        q_string = this.serialize(r_arguments);
        signature = this.get_signature(this.stringify_request(r_arguments));
        url = this.domain + 'register' + '?' + 'signature=' + signature + ((q_string !== '') ? ('&' + q_string) : '');
        console.log(url)
        return url;
    }

    /**
     * Generates signature based on api_key, secret_key and query params that is needed for each request
     * @param String query_params GET query parameter that is being sent to the server
     * @returns String the signature that is needed to be attach along with the request
     */
    this.get_signature = function (request_string) {
        var api_key = 'f5dd7eaf67ec1451ad7789d4963dd7238bbeb608';
        var secret_key = '7d14768e74713ba6c81c38e485076d263da79702';
        var signature = hex_sha1(hex_md5(api_key) + hex_md5(secret_key) + request_string);
        return signature;
    };

    /**
     * Convert Request Object to String. Request Object will be sorted before being converted.
     * @param Object request Request Object to be converted.
     * @returns String stringified Request Object.
     */
    this.stringify_request = function (request) {
        var str = [];
        var request_string;
        for (var p in request)
            if (request.hasOwnProperty(p))
                str.push(p + "=" + request[p]);
        str.alphanumSort(true);
        return str.join("~");
    };

    /**
     * Converts JSON type data to GET query parameter format.
     * @param Object obj JSON data to be converted
     * @returns String GET query parameter equivalent of given JSON data
     */
    this.serialize = function (obj) {
        var str = [];
        for (var p in obj)
            if (obj.hasOwnProperty(p))
                str.push(encodeURIComponent(p) + "=" + encodeURIComponent(obj[p]));
        return str.join("&");
    };

    /**
     * Creates a new copy of given obj to avoid pass by reference
     * @param Object obj Object to copy
     * @returns Object the new copy of given obj.
     */
    this.clone_object = function (obj) {
            var n_obj = new Object();
            for (var p in obj)
                if (obj.hasOwnProperty(p))
                    n_obj[p] = obj[p]

            return n_obj;
        }
        /**
         * Merges two given objects.
         * @param Object obj1 first object.
         * @param Object obj2 first object.
         * @returns Object a new object based on merged obj1 and obj2.
         */
    this.merge_objects = function (obj1, obj2) {
        var obj3 = {};
        for (var p in obj1)
            if (obj1.hasOwnProperty(p))
                obj3[p] = obj1[p];
        for (var p in obj2)
            if (obj2.hasOwnProperty(p))
                obj3[p] = obj2[p];
        return obj3;
    }

    this.trans_from_rqst = function (obj) {
        var str = [];
        for (var p in obj)
            str.push(encodeURIComponent(p) + "=" + encodeURIComponent(obj[p]));
        return str.join("&");
    }


    this.http_options = {
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
        },
        transformRequest: this.trans_from_rqst,
        //timeout: 10000
    }

    /**
     *  Retrieve a record of the specified model with respect to given id and argument and submit it as data to the specified callback function.
     *  @param string model name of model to retrieve.
     *  @param string id that will be used as primary identifier to retrieve specified model.
     *  @param object arguments list of key-value pair to change behavior of retrieving data.
     *  @param function callback function to be excecuted when api call was finished. Has a parameter that will hold the response object.
     *  @return null
     */

    this.get = function (model, id, arguments, callback, dependency) {
        var response = new Object();
        var q_string;
        var r_arguments;
        var url;
        var signature;

        if (dependency !== undefined) {
            r_arguments = this.merge_objects(arguments, dependency);
        } else {
            r_arguments = this.clone_object(arguments);
        }
        q_string = this.serialize(r_arguments);
        r_arguments.id = id;
        signature = this.get_signature(this.stringify_request(r_arguments));
        url = this.api_url + model + '/' + id + '?' + 'signature=' + signature + ((q_string !== '') ? ('&' + q_string) : '');

        $http.get(url).
        success(function (data, status, headers, config) {
            response.data = data;
            response.status = 200;
            response.message = 'Request was successful.';
            callback(response);
        }).
        error(function (data, status, headers, config) {
            response.data = null;
            response.status = status;
            response.code = data.code;
            response.message = data.message;
            callback(response);
        });
        return null;
    };

    /**
     *  Retrieve all record of the specified model with respect to arguments and submit it as data to the specified callback function.
     *  @param string model name of model to retrieve.
     *  @param object conditions list of key-value pair to be used as condition field name values.
     *  @param object arguments list of key-value pair to change behavior of retrieving data.
     *  @param function callback function to be excecuted when api call was finished. Has a parameter that will hold the response object.
     *  @return null
     */
    this.get_all = function (model, conditions, arguments, callback, error_callback, dependency) {
        var response = new Object();
        var r_condition = new Object();
        var r_arguments;
        var q_string;
        var signature;
        var url;
        var c = 1;

        for (var condition in conditions) {
            if (conditions.hasOwnProperty(condition)) {
                r_condition['condition' + c + '-' + condition] = conditions[condition];
            }
            c++;
        }

        if (dependency !== undefined) {
            r_arguments = this.merge_objects(this.merge_objects(arguments, r_condition), dependency);
        } else {
            r_arguments = this.merge_objects(arguments, r_condition);
        }

        q_string = this.serialize(r_arguments);
        signature = this.get_signature(this.stringify_request(r_arguments));
        url = this.api_url + model + '?' + 'signature=' + signature + ((q_string !== '') ? ('&' + q_string) : '');


        $http.get(url, this.http_options).
        success(function (data, status, headers, config) {
            response.data = data;
            response.status = 200;
            response.message = 'Request was successful.';
            callback(response);
        }).
        error(function (data, status, headers, config) {
            response.data = null;
            response.status = status;
            response.message = data.statusText;
            callback(response);
        });

        return null;
    };

    /**
     *  Create a record of the specified model with respect to arguments and submit the response as data to the specified callback function.
     *  @param string model name of model to be created.
     *  @param object arguments list of key-value pair to be used as field name values.
     *  @param function callback function to be excecuted when api call was finished. Has a parameter that will hold the response object.
     *  @return null
     */
    this.create = function (model, arguments, callback, dependency) {
        var response = new Object();
        var r_arguments;
        var url;
        var signature;

        if (dependency !== undefined) {
            r_arguments = this.merge_objects(arguments, dependency);
        } else {
            r_arguments = this.clone_object(arguments);
        }

        q_string = this.serialize(dependency);
        signature = this.get_signature(this.stringify_request(r_arguments));
        url = this.api_url + model + '?' + 'signature=' + signature + ((q_string !== '') ? ('&' + q_string) : '');

        var canceler = $q.defer();
        $http.post(url, arguments, this.http_options).
        success(function (data, status, headers, config) {
            response.data = data;
            response.status = 201;
            response.message = 'Request was successful.';
            callback(response);
        }).
        error(function (data, status, headers, config) {
            response.data = data;
            response.status = status;
            response.code = data.code;
            response.message = data.message;
            response.details = data.responseText;
            callback(response);
        });


        return null;
    };

    /**
     *  Update a record of the specified model with the given id with respect to arguments and submit the response as data to the specified callback function.
     *  @param string model name of model to update.
     *  @param string id that will be used as primary identifier to update specified model.
     *  @param object arguments list of key-value pair to be used as field name values.
     *  @param function callback function to be excecuted when api call was finished. Has a parameter that will hold the response object.
     */
    this.update = function (model, id, arguments, callback, dependency) {
        var response = new Object();
        var r_arguments;
        var url;
        var signature;

        if (dependency !== undefined) {
            r_arguments = this.merge_objects(arguments, dependency);
        } else {
            r_arguments = this.clone_object(arguments);
        }

        r_arguments.id = id;
        q_string = this.serialize(dependency);
        signature = this.get_signature(this.stringify_request(r_arguments));
        url = this.api_url + model + '/' + id + '?' + 'signature=' + signature + ((q_string !== '') ? ('&' + q_string) : '');

        $http.put(url, arguments, this.http_options).
        success(function (data, status, headers, config) {
            response.data = data;
            response.status = 204;
            response.message = 'Request was successful.';
            callback(response);
        }).
        error(function (data, status, headers, config) {
            response.data = data;
            response.status = status;
            response.code = data.code;
            response.message = data.message;
            response.details = data.responseText;
            callback(response);
        });

        return null;
    };

    /**
     *  Update all records of the specified model with the given condition with respect to arguments and submit the response as data to the specified callback function.
     *  @param string model name of model to update.
     *  @param object conditions list of key-value pair to be used as condition field name values.
     *  @param object arguments list of key-value pair to be used as field name values.
     *  @param function callback function to be excecuted when api call was finished. Has a parameter that will hold the response object.
     */
    this.update_all = function (model, conditions, arguments, callback, error_callback, dependency) {
        var response = new Object();
        var r_condition = new Object();
        var r_arguments;
        var signature;
        var url;
        var c = 1;

        for (var condition in conditions) {
            if (conditions.hasOwnProperty(condition)) {
                r_condition['condition' + c + '-' + condition] = conditions[condition];
            }
            c++;
        }

        if (dependency !== undefined) {
            r_arguments = this.merge_objects(this.merge_objects(arguments, r_condition), dependency);
        } else {
            r_arguments = this.merge_objects(arguments, r_condition);
        }

        q_string = this.serialize(dependency);
        signature = this.get_signature(this.stringify_request(r_arguments));
        url = this.api_url + model + '?' + 'signature=' + signature + ((q_string !== '') ? ('&' + q_string) : '');

        $http.put(url, r_arguments, this.http_options).
        success(function (data, status, headers, config) {
            response.data = data;
            response.status = 203;
            response.message = 'Request was successful.';
            callback(response);
        }).
        error(function (data, status, headers, config) {
            response.data = data;
            response.status = status;
            response.code = data.code;
            response.message = data.message;
            response.details = data.responseText;
            callback(response);
        });

        return null;
    };


    /**
     *  Delete a record of the specified model with the given id and submit the response as data to the specified callback function.
     *  @param string model name of model to delete.
     *  @param string id that will be used as primary identified to delete specified model. Set to null to delete all records of the model.
     *  @param function callback function to be excecuted when api call was finished. Has a parameter that will hold the response object.
     */
    this.delete = function (model, id, callback, dependency) {
        var response = new Object();
        var r_arguments;
        var url;
        var signature;

        if (dependency !== undefined) {
            r_arguments = dependency;
        } else {
            r_arguments = {};
        }

        r_arguments.id = id;
        q_string = this.serialize(dependency);
        signature = this.get_signature(this.stringify_request(r_arguments));
        url = this.api_url + model + '/' + id + '?' + 'signature=' + signature + ((q_string !== '') ? ('&' + q_string) : '');

        $http.delete(url, {
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
            },
            transformRequest: this.transfromRqst
        }).
        success(function (data, status, headers, config) {
            response.data = data;
            response.status = 202;
            response.message = 'Request was successful.';
            callback(response);
        }).
        error(function (data, status, headers, config) {
            response.data = data;
            response.status = status;
            response.code = data.code;
            response.message = data.message;
            response.details = data.responseText;
            callback(response);
        });

        return null;
    };

    /**
     *  Delete all records of the specified model with the given condition and submit the response as data to the specified callback function.
     *  @param string model name of model to delete.
     *  @param object conditions list of key-value pair to be used as condition field name values.
     *  @param function callback function to be excecuted when api call was finished. Has a parameter that will hold the response object.
     */
    this.delete_all = function (model, conditions, callback, error_callback, dependency) {
        var response = new Object();
        var r_condition = new Object();
        var r_arguments;
        var signature;
        var url;
        var c = 1;

        for (var condition in conditions) {
            if (conditions.hasOwnProperty(condition)) {
                r_condition['condition' + c + '-' + condition] = conditions[condition];
            }
            c++;
        }

        if (dependency !== undefined) {
            r_arguments = this.merge_objects(this.merge_objects(arguments, r_condition), dependency);
        } else {
            r_arguments = this.merge_objects(arguments, r_condition);
        }

        q_string = this.serialize(dependency);
        signature = this.get_signature(this.stringify_request(r_arguments));
        url = this.api_url + model + '?' + 'signature=' + signature + ((q_string !== '') ? ('&' + q_string) : '');

        $http.delete(url, r_condition, this.http_options).
        success(function (data, status, headers, config) {
            response.data = data;
            response.status = 202;
            response.message = 'Request was successful.';
            callback(response);
        }).
        error(function (data, status, headers, config) {
            response.data = data;
            response.status = status;
            response.code = data.code;
            response.message = data.message;
            response.details = data.responseText;
            callback(response);
        });


        return null;
    };

};